@@ -76,7 +76,6 @@ cv_varsel <- function(fit,  method = NULL, cv_method = NULL,
 	cv_method <- args$cv_method
 	K <- args$K
 
-
 	# search options
 	opt <- list(lambda_min_ratio=lambda_min_ratio, nlambda=nlambda, thresh=thresh, regul=regul)
 	
@@ -90,12 +89,12 @@ cv_varsel <- function(fit,  method = NULL, cv_method = NULL,
 		sel_cv <- loo_varsel(refmodel, method, nv_max, ns, nc, nspred, ncpred, relax, intercept, penalty, 
 		                     verbose, opt, nloo = nloo, validate_search = validate_search, seed = seed)
 	} else {
-		stop(sprintf('Unknown cross-validation method: %s.', method))
+               stop(sprintf('Unknown cross-validation method: %s.', cv_method))
 	}
 	
 	# run the selection using the full dataset
 	if (verbose)
-		print(paste('Performing the selection using all the data..'))
+		cat('Performing variable selection using all data...\n')
 	sel <- varsel(refmodel, method=method, ns=ns, nc=nc, nspred=nspred, ncpred=ncpred,
 	              relax=relax, nv_max=nv_max, intercept=intercept, penalty=penalty, verbose=verbose, 
 	              lambda_min_ratio=lambda_min_ratio, nlambda=nlambda, regul=regul)
@@ -118,6 +117,8 @@ cv_varsel <- function(fit,  method = NULL, cv_method = NULL,
 	vs <- list()
 	vs$refmodel <- refmodel
 	vs$spath <- sel$spath
+	vs$method <- method
+	vs$cv_method <- cv_method
 	vs <- c(vs, c(sel_cv[c('d_test', 'summaries')],
 	              sel[c('family_kl', 'vind', 'kl')],
 	              list(pctch = pctch)))
@@ -125,9 +126,6 @@ cv_varsel <- function(fit,  method = NULL, cv_method = NULL,
 	vs$nv_max <- nv_max
 	vs$nv_all <- ncol(refmodel$x)
 	vs$ssize <- suggest_size(vs, warnings = F)
-	
-	if (verbose)
-	  print('Done.')
 
 	vs
 }
@@ -147,11 +145,11 @@ parseargs_cv_varsel <- function(refmodel, cv_method, K) {
     else
       cv_method <- 'LOO'
   }
-  if (cv_method == 'kfold' && is.null(K)) {
-    if ('datafit' %in% class(refmodel))
-      K <- 10
+  if (cv_method == 'kfold') {
+    if (is.null(K))
+      K <- if ('datafit' %in% class(refmodel)) 10 else 5
     else
-      K <- 5
+      .validate_num_folds(K, refmodel$nobs)
   }
   
   list(cv_method=cv_method, K=K)
@@ -195,7 +193,7 @@ kfold_varsel <- function(refmodel, method, nv_max, ns, nc, nspred, ncpred, relax
   
   # Perform the selection for each of the K folds
   if (verbose) {
-    print('Performing selection for each fold..')
+    cat('Performing selection for each fold...\n')
     pb <- utils::txtProgressBar(min = 0, max = K, style = 3, initial=0)
   }
   spath_cv <- lapply(seq_along(list_cv), function(fold_index) {
@@ -213,7 +211,7 @@ kfold_varsel <- function(refmodel, method, nv_max, ns, nc, nspred, ncpred, relax
   # Construct submodel projections for each fold
   as.search <- !relax && !is.null(spath_cv[[1]]$beta) && !is.null(spath_cv[[1]]$alpha)
   if (verbose && !as.search) {
-    print('Computing projections..')
+    cat('Computing projections...\n')
     pb <- utils::txtProgressBar(min = 0, max = K, style = 3, initial=0)
   }
   p_sub_cv <- mapply(function(spath, fold_index) {
@@ -273,7 +271,7 @@ kfold_varsel <- function(refmodel, method, nv_max, ns, nc, nspred, ncpred, relax
       # is datafit, cvfun will return an empty list and this will lead to normal cross-validation
       # for the submodels although we don't have an actual reference model
       if (verbose && !('datafit' %in% class(refmodel)))
-        print('Performing cross-validation for the reference model..')
+        cat('Performing cross-validation for the reference model...\n')
       folds <- cvfolds(refmodel$nobs, k=K, seed=seed)
       cvfits <- refmodel$cvfun(folds)
       cvfits <- lapply(seq_along(cvfits), function(k) {
@@ -315,6 +313,12 @@ loo_varsel <- function(refmodel, method, nv_max, ns, nc, nspred, ncpred, relax,
 	fam <- refmodel$fam
 	mu <- refmodel$mu
 	dis <- refmodel$dis
+	n <- nrow(mu)
+
+	# by default use all observations
+	nloo <- ifelse(is.null(nloo), n, min(nloo, n))
+	if (nloo < 1)
+		stop('Value of \'nloo\' must be at least 1')
 
 	# training data
 	d_train <- .get_traindata(refmodel)
@@ -339,9 +343,6 @@ loo_varsel <- function(refmodel, method, nv_max, ns, nc, nspred, ncpred, relax,
 	psisloo <- loo::psis(-loglik, cores = 1, r_eff = rep(1,ncol(loglik))) # TODO: should take r_eff:s into account
 	lw <- weights(psisloo)
 	pareto_k <- loo::pareto_k_values(psisloo)
-	n <- length(pareto_k)
-	nloo <- ifelse(is.null(nloo), n, nloo) # by default use all observations
-	nloo <- min(nloo,n)
 
 	# compute loo summaries for the reference model
 	d_test <- d_train
@@ -360,8 +361,8 @@ loo_varsel <- function(refmodel, method, nv_max, ns, nc, nspred, ncpred, relax,
 	mu_sub <- matrix(nrow=n, ncol=nv_max+1)
 
 	if (verbose) {
-    print('Computing LOOs...')
-    pb <- utils::txtProgressBar(min = 0, max = nloo, style = 3, initial=0)
+		cat('Computing LOOs...\n')
+		pb <- utils::txtProgressBar(min = 0, max = nloo, style = 3, initial=0)
 	}
 
 	if (!validate_search) {
@@ -474,9 +475,3 @@ loo_varsel <- function(refmodel, method, nv_max, ns, nc, nspred, ncpred, relax,
   return(list(inds=inds, w=w))
   
 }
-
-
-
-
-
-
