@@ -11,26 +11,44 @@ using namespace Rcpp;
 using namespace arma;
 
 
+
+/**
+ * Returns the value of the penalty term in the elastic net regularization.
+ */
+double elnet_penalty(vec beta, // coefficients
+                     double lambda, // regularization parameter
+                     double alpha, // elastic net mixing parameter
+                     vec penalty) // relative penalties for the variables 
+{
+  double value;
+  uvec fin = find_finite(penalty);
+  value = lambda*sum(penalty.elem(fin) % (0.5*(1-alpha)*square(beta.elem(fin))
+                                            + alpha*(abs(beta.elem(fin))) ) );
+  return(value);
+}
+
+
 /** Returns the value of the regularized quadratic approximation to the loss function
 that is to be minimized iteratively:
 L = 0.5*sum_i{ w_i*(z_i - f_i)^2 } + lambda*{ 0.5*(1-alpha)*||beta||_2^2 + alpha*||beta||_1 }
 */
-double loss_approx(vec beta,    // coefficients
-                   vec f,       // latent values
-                   vec z,       // locations for pseudo obsevations
-                   vec w,       // weights of the pseudo observations (inverse-variances)
+double loss_approx(vec& beta,    // coefficients
+                   vec& f,       // latent values
+                   vec& z,       // locations for pseudo obsevations
+                   vec& w,       // weights of the pseudo observations (inverse-variances)
                    double lambda, // regularization parameter
                    double alpha, // elastic net mixing parameter
-                   vec penalty) // relative penalties for the variables  
+                   vec& penalty) // relative penalties for the variables  
 {
   double loss;
   uvec fin = find_finite(penalty);
-  loss = 0.5*sum(w % square(z-f)) + 
-    lambda*sum(penalty.elem(fin) % (0.5*(1-alpha)*square(beta.elem(fin)) + alpha*(abs(beta.elem(fin)))));
+  loss = 0.5*sum(w % square(z-f)) + elnet_penalty(beta,lambda,alpha,penalty);
   return loss;
 }
 
 
+
+
 /** Updates the regression coefficients and the intercept (unless excluded) based on the
 current quadratic approximation to the loss function. This is done via the 'soft-thresholding'
 as described by Friedman et. al (2009). Performs either one pass through the specified set
@@ -104,8 +122,6 @@ void coord_descent(	vec& beta, // regression coefficients
     loss = loss_approx(beta,f,z,w,lambda,alpha,penalty);
     
     if (until_convergence) {
-      // Rcpp::Rcout << "loss - loss_old = " << loss - loss_old << '\n';
-      // if (fabs(loss_old-loss) < tol) {
       if (loss_old-loss < tol) {
         break;
       } else {
@@ -163,6 +179,7 @@ List glm_elnet_c(arma::mat x, // input matrix
   // for storing the whole solution path
   rowvec beta0_path(nlam);
   mat beta_path(D,nlam);
+  mat w_path(x.n_rows,nlam);
   beta0_path.zeros();
   beta_path.zeros();
   int npasses = 0; // counts how many times the coefficient vector is looped through
@@ -185,7 +202,6 @@ List glm_elnet_c(arma::mat x, // input matrix
   obs = pseudo_obs(f,w0);
   z = as<vec>(obs["z"]);
   w = as<vec>(obs["w"]);
-  // double loss_initial = obs["dev"];
   double loss_initial = loss_approx(beta, f, z, w, lambda(0), alpha, penalty); // initial loss
   double loss_old = loss_initial; // will be updated iteratively
   double loss; // will be updated iteratively
@@ -207,6 +223,7 @@ List glm_elnet_c(arma::mat x, // input matrix
       
       // current value of the (approximate) loss function
       loss_old = loss_approx(beta, f, z, w, lam, alpha, penalty);
+      // loss_old = ((double) obs["loss"]) + elnet_penalty(beta, lam, alpha, penalty);
       
       // run the coordinate descent until convergence for the current
       // quadratic approximation
@@ -233,9 +250,12 @@ List glm_elnet_c(arma::mat x, // input matrix
       
       // the loss after updating the coefficients
       loss = loss_approx(beta, f, z, w, lam, alpha, penalty);
-      
+      // obs = pseudo_obs(f,w);
+      // loss = ((double) obs["loss"]) + elnet_penalty(beta, lam, alpha, penalty);
+
       // check if converged
       if (fabs(loss_old-loss) < tol) {
+      // if (loss_old-loss < tol) {
         // convergence reached; proceed to the next lambda value
         break;
       }
@@ -243,6 +263,7 @@ List glm_elnet_c(arma::mat x, // input matrix
     // store the current solution
     beta0_path(k) = beta0;
     beta_path.col(k) = beta;
+    w_path.col(k) = w;
     qa_updates(k) = qau;
     
     if (qau == qa_updates_max && qa_updates_max > 1)
@@ -263,7 +284,7 @@ List glm_elnet_c(arma::mat x, // input matrix
     }
   }
   
-  return List::create(beta_path, beta0_path, npasses, qa_updates, as_updates);
+  return List::create(beta_path, beta0_path, w_path, npasses, qa_updates, as_updates);
 }
 
 
@@ -281,15 +302,18 @@ void glm_ridge( vec& beta,      // output: regression coefficients (contains int
                 Function pseudo_obs,
                 double lambda,
                 bool intercept,
+                arma::vec penalty, // relative penalties for the variables
                 double thresh,
                 int qa_updates_max,
-                int ls_iter_max=20,
+                int ls_iter_max=50,
                 bool debug=false)
 {
   
-  if (intercept)
-    // add a vector of ones to x
+  if (intercept) {
+    // add a vector of ones to x and set zero penalty for the intercept
     x = join_horiz(ones<vec>(x.n_rows), x);
+    penalty = join_vert(zeros<vec>(1), penalty); 
+  }
   
   int n = x.n_rows;
   int D = x.n_cols;
@@ -300,28 +324,26 @@ void glm_ridge( vec& beta,      // output: regression coefficients (contains int
   double b = 0.5; 
   
   // initialization
-  // beta.zeros();
   vec beta_new(D); beta_new.zeros();
   vec dbeta(D); dbeta.zeros();
-  vec grad(D); grad.zeros(); // gradient of the deviance w.r.t. the regression coefficients
-  vec grad_f(n); grad_f.zeros(); // pointwise gradient of the deviance w.r.t. f
+  vec grad(D); grad.zeros(); // gradient of the negative log likelihood w.r.t. the regression coefficients
+  vec grad_f(n); grad_f.zeros(); // pointwise gradient of the negative log likelihood w.r.t. the latent values f
   vec f = x*beta;
   
   mat xw(n,D); // this will be the weighted x
-  mat regmat = lambda*eye(D,D); // regularization matrix
+  mat regmat = lambda*diagmat(penalty);//eye(D,D); // regularization matrix
   
   // initial quadratic approximation
   List obs = pseudo_obs(f,w);
   vec z = as<vec>(obs["z"]);
   w = as<vec>(obs["w"]);
   grad_f = as<vec>(obs["grad"]);
-  double loss_initial = obs["dev"];
+  double loss_initial = ((double) obs["loss"]) + elnet_penalty(beta, lambda, 0, penalty);
   double loss_old = loss_initial; // will be updated iteratively
   loss = loss_initial; // will be updated iteratively
   double tol = thresh*fabs(loss_initial); // threshold for convergence
   double decrement = 0; // newton decrement, used to monitor convergence
   
-  
   qau = 0;
   while (qau < qa_updates_max) {
     
@@ -330,9 +352,9 @@ void glm_ridge( vec& beta,      // output: regression coefficients (contains int
       xw.col(j) = x.col(j) % sqrt(w);
     
     // weighted least squares solution
-    beta_new = solve( xw.t()*xw + regmat, xw.t()*(sqrt(w)%z) );
+    beta_new = solve( xw.t()*xw + regmat, xw.t()*(sqrt(w)%z) ); 
     dbeta = beta_new - beta;
-    grad = x.t()*grad_f + 2*lambda*beta; // grad of deviance + grad of penalty
+    grad = x.t()*grad_f + lambda*penalty%beta; // gradient of negative log likelihood + gradient of penalty
     decrement = -sum(grad%dbeta); // newton decrement
     
     // check for convergence
@@ -349,8 +371,7 @@ void glm_ridge( vec& beta,      // output: regression coefficients (contains int
       t = b*t;
       f = x*(beta+t*dbeta);
       obs = pseudo_obs(f,w);
-      loss = obs["dev"];
-      loss = loss + lambda*sum(square(beta+t*dbeta));
+      loss = ((double) obs["loss"]) + elnet_penalty(beta+t*dbeta, lambda, 0, penalty);
       ++ls_iter;
       
       if (std::isnan(loss))
@@ -367,7 +388,16 @@ void glm_ridge( vec& beta,      // output: regression coefficients (contains int
     }
     
     if (ls_iter == ls_iter_max && ls_iter_max > 1) {
+      // beta.print("beta = ");
+      // dbeta.print("dbeta = ");
+      // grad.print("grad = ");
+      // Rcpp::Rcout << "loss = " << loss << "\n";
+      // Rcpp::Rcout << "loss_initial = " << loss_initial << "\n";
+      // Rcpp::Rcout << "tol = " << tol << "\n";
+      // Rcpp::Rcout << "decrement = " << decrement << "\n";
+      // Rcpp::Rcout << "\n\n";
       Rcpp::Rcout << "glm_ridge warning: maximum number of line search iterations reached. The optimization can be ill-behaved.\n";
+      break;
     }
     
     // update the solution
@@ -399,7 +429,8 @@ List glm_ridge_c( arma::mat x,
                   Function pseudo_obs,
                   double lambda,
                   bool intercept,
-                  arma::vec beta_init, // initial value for the latent values (containing the intercept as the first element)
+                  arma::vec penalty, // relative penalties for the variables
+                  arma::vec beta_init, // initial value for the coefficients (containing the intercept as the first element)
                   arma::vec w_init, // initial guess for the weights of the pseudo-gaussian observations (needed for Student-t model)
                   double thresh,
                   int qa_updates_max,
@@ -414,12 +445,14 @@ List glm_ridge_c( arma::mat x,
   vec w = w_init;
   int qau;
   double loss;
-  glm_ridge(beta, loss, w, qau, x, pseudo_obs, lambda, intercept, thresh, qa_updates_max, ls_iter_max, debug);
+  glm_ridge(beta, loss, w, qau, x, pseudo_obs, lambda, intercept, penalty, thresh, qa_updates_max, ls_iter_max, debug);
     
   if (intercept) 
-    return List::create(vec(beta.tail(D-1)), beta(0), w, qau);
+    return List::create(vec(beta.tail(D-1)), beta(0), w, loss, qau);
+    // return List::create(vec(beta.tail(D-1)), beta(0), w, qau);
   else 
-    return List::create(beta, 0.0, w, qau);
+    return List::create(beta, 0.0, w, loss, qau);
+    // return List::create(beta, 0.0, w, qau);
 }
 
 
@@ -434,11 +467,12 @@ List glm_forward_c( arma::mat x, // inputs (features)
                     Function pseudo_obs, // R-function returning the pseudo-data based on the quadratic approximation
                     double lambda, // regularization parameter (multiplier for L2-penalty)
                     bool intercept, // whether to use intercept
+                    arma::vec penalty, // relative penalties for the variables
                     double thresh, // threshold for stopping the iterative reweighted least squares
                     int qa_updates_max, // max number or quadratic approximation updates
                     int pmax, // maximum number of variables up to which the search is continued
                     arma::vec w0, // initial guess for the weights of the pseudo-gaussian observations (needed for Student-t model)
-                    int ls_iter_max=100 ) // max number of line search iterations
+                    int ls_iter_max=50 ) // max number of line search iterations
 {
   
   mat xp; // x for the current active set
@@ -449,6 +483,7 @@ List glm_forward_c( arma::mat x, // inputs (features)
   uvec varorder(pmaxu); varorder.zeros(); // stores the order in which the variables are added to the model
   mat beta_all(D,pmaxu); beta_all.zeros(); // collects beta from all steps
   rowvec beta0_all(pmaxu); beta0_all.zeros(); // collects beta0 from all steps
+  mat w_all(x.n_rows,pmaxu); // collects weights of the gaussian pseudo-observations from all steps
   
   // declare a few variables that are needed during the iteration
   vec w = w0;
@@ -473,7 +508,8 @@ List glm_forward_c( arma::mat x, // inputs (features)
       
       chosen(j) = 1;
       beta.zeros();
-      glm_ridge(beta, loss, w, qau, x.cols(find(chosen)), pseudo_obs, lambda, intercept, thresh, qa_updates_max, ls_iter_max);
+      glm_ridge(beta, loss, w, qau, x.cols(find(chosen)), pseudo_obs, lambda, intercept, 
+                penalty.elem(find(chosen)), thresh, qa_updates_max, ls_iter_max);
       chosen(j) = 0;
       
       if (loss < loss_min) {
@@ -494,9 +530,10 @@ List glm_forward_c( arma::mat x, // inputs (features)
       beta0_all(k) = 0;
       beta_all.submat(find(chosen), step) = betaopt;
     }
+    w_all.col(k) = w;
   }
   
-  return List::create(beta_all, beta0_all, varorder);
+  return List::create(beta_all, beta0_all, varorder, w_all);
 }
 
 
